#include "polynomial.hpp"

#include <utility>

ConstSizePolynomial::ConstSizePolynomial(size_t degree)
: degree_(degree)
, coeffs_(new float[degree_+1])
{
    for(auto i = 0; i < degree_+1; ++i)
    {
        coeffs_[i] = 0.0f;
    }
}
    
ConstSizePolynomial::ConstSizePolynomial(const ConstSizePolynomial& other)
: ConstSizePolynomial(other.degree_)
{
    for(auto i = 0; i < degree_+1; ++i)
    {
        coeffs_[i] = other.coeffs_[i];
    }
}

ConstSizePolynomial::ConstSizePolynomial(ConstSizePolynomial&& other)
: degree_(other.degree_)
, coeffs_(std::move(other.coeffs_)) 
{
    other.coeffs_ = nullptr;
}

ConstSizePolynomial& ConstSizePolynomial::operator=(ConstSizePolynomial&& other)
{
    std::swap(degree_, other.degree_);
    std::swap(coeffs_, other.coeffs_);

    return *this;
}

ConstSizePolynomial::~ConstSizePolynomial()
{
    if(nullptr != coeffs_)
    {
        delete[] coeffs_;
    }
}

void ConstSizePolynomial::setCoeff(size_t n, float coeff)
{
    if( n <= degree_)
    {
        coeffs_[n] = coeff;
    }
}

float ConstSizePolynomial::value(float arg) const
{
    float result = 0.0f;

    for(auto i = degree_+1; i; --i)
    {
        result = result * arg + coeffs_[i-1];
    }

    return result;
}

float ConstSizePolynomial::coeff(size_t n) const
{
    if (n <= degree_)
    {
        return coeffs_[n];
    }
    else
    {
        return 0.0f;
    }
}

size_t ConstSizePolynomial::degree() const
{
    size_t degree = 0;

    for(auto i = 0; i <= degree_; ++i)
    {
        if (0.0f != coeffs_[i]) 
        {
            degree = i;
        }
    } 

    return degree;
}


Polynomial::Polynomial()
: currentPolynomialDegree_(0)
, polynomial_(currentPolynomialDegree_)
{ 
}

void Polynomial::setCoeff(size_t n, float coeff)
{
    if( n > currentPolynomialDegree_)
    {
        _resize(n);
    }

    polynomial_.setCoeff(n, coeff);
}

float Polynomial::value(float arg) const
{
    return polynomial_.value(arg);
}

float Polynomial::coeff(size_t n) const
{
    return polynomial_.coeff(n);
}

size_t Polynomial::degree() const
{
    return polynomial_.degree();
}

void Polynomial::_resize(size_t degree)
{
    if(degree <= polynomial_.degree())
    {
        return;
    }

    auto polynomial = ConstSizePolynomial(degree);
    for(auto i = 0; i <= polynomial_.degree(); ++i )
    {
        polynomial.setCoeff(i, polynomial_.coeff(i));
    }

    polynomial_ = std::move(polynomial);
    currentPolynomialDegree_ = degree;
}
