// #include "polynomial.hpp"

#include <utility>

template<typename T>
ConstSizePolynomial<T>::ConstSizePolynomial(size_t degree)
: degree_(degree)
, coeffs_(new T[degree_+1])
{
    for(auto i = 0; i < degree_+1; ++i)
    {
        coeffs_[i] = static_cast<T>(0);
    }
}
    
template<typename T>
ConstSizePolynomial<T>::ConstSizePolynomial(const ConstSizePolynomial& other)
: ConstSizePolynomial(other.degree_)
{
    for(auto i = 0; i < degree_+1; ++i)
    {
        coeffs_[i] = other.coeffs_[i];
    }
}

template<typename T>
ConstSizePolynomial<T>::ConstSizePolynomial(ConstSizePolynomial&& other)
: degree_(other.degree_)
, coeffs_(std::move(other.coeffs_)) 
{
    other.coeffs_ = nullptr;
}

template<typename T>
ConstSizePolynomial<T>& ConstSizePolynomial<T>::operator=(ConstSizePolynomial<T>&& other)
{
    std::swap(degree_, other.degree_);
    std::swap(coeffs_, other.coeffs_);

    return *this;
}

template<typename T>
ConstSizePolynomial<T>::~ConstSizePolynomial()
{
    if(nullptr != coeffs_)
    {
        delete[] coeffs_;
    }
}

template<typename T>
void ConstSizePolynomial<T>::setCoeff(size_t n, const T& coeff)
{
    if( n <= degree_)
    {
        coeffs_[n] = coeff;
    }
}

template<typename T>
T ConstSizePolynomial<T>::value(const T& arg) const
{
    float result = 0.0f;

    for(auto i = degree_+1; i; --i)
    {
        result = result * arg + coeffs_[i-1];
    }

    return result;
}

template<typename T>
T ConstSizePolynomial<T>::coeff(size_t n) const
{
    if (n <= degree_)
    {
        return coeffs_[n];
    }
    else
    {
        return static_cast<T>(0);
    }
}

template<typename T>
size_t ConstSizePolynomial<T>::degree() const
{
    size_t degree = 0;

    for(auto i = 0; i <= degree_; ++i)
    {
        if (static_cast<T>(0) != coeffs_[i]) 
        {
            degree = i;
        }
    } 

    return degree;
}

template<typename T>
Polynomial<T>::Polynomial()
: currentPolynomialDegree_(0)
, polynomial_(currentPolynomialDegree_)
{ 
}

template<typename T>
void Polynomial<T>::setCoeff(size_t n, const T& coeff)
{
    if( n > currentPolynomialDegree_)
    {
        _resize(n);
    }

    polynomial_.setCoeff(n, coeff);
}

template<typename T>
T Polynomial<T>::value(const T& arg) const
{
    return polynomial_.value(arg);
}

template<typename T>
T Polynomial<T>::coeff(size_t n) const
{
    return polynomial_.coeff(n);
}

template<typename T>
size_t Polynomial<T>::degree() const
{
    return polynomial_.degree();
}

template<typename T>
void Polynomial<T>::_resize(size_t degree)
{
    if(degree <= polynomial_.degree())
    {
        return;
    }

    auto polynomial = ConstSizePolynomial<T>(degree);
    for(auto i = 0; i <= polynomial_.degree(); ++i )
    {
        polynomial.setCoeff(i, polynomial_.coeff(i));
    }

    polynomial_ = std::move(polynomial);
    currentPolynomialDegree_ = degree;
}
